# Copyright (c) 2009 gocept gmbh & co. kg
# See also LICENSE.txt

import ConfigParser
import datetime
import email.mime.text
import gocept.support.staff
import logging
import logging.config
import optparse
import os
import sys
import urllib
import urllib2
import zope.sendmail.mailer


log = logging.getLogger(__name__)


class Notifications(object):

    # List all pagers, emails and phones that we want to notify
    pagers = ()
    emails = ()
    phones = ()

    def __init__(self, config):
        self.config = config
        self.emails = []
        self.phones = []
        self.pagers = []

        self._load_contacts()

    @classmethod
    def fromMain(cls):
        parser = optparse.OptionParser()
        parser.add_option('-c', '--config', default='~/.gocept-support.cfg')
        options, args = parser.parse_args()
        config = ConfigParser.ConfigParser()
        config.read([os.path.expanduser(options.config)])
        logging.basicConfig()
        return Notifications(config), options, args

    def _load_contacts(self):
        schedule = gocept.support.staff.Staff(
            self.config.get('main', 'staff'))
        for assignment in schedule.assignments(datetime.date.today()):
            if assignment.email:
                self.emails.append(assignment.email)
            if assignment.mobile:
                self.phones.append(assignment.mobile)
            if assignment.pager:
                self.pagers.append(assignment.pager)

    def notify(self, subject, message):
        """Notify the support about an important incident.

        Go to extreme measures to ensure that we get the information out.

        XXX provide escalation mechanism if this fails

        """
        log.info('Sending notification %s', subject)
        # notify per pager
        for pager in self.pagers:
            try:
                self.notify_cityruf(pager, subject)
            except:
                log.error(
                    'Pager notification failed: %s [%s]',
                    pager, subject, exc_info=True)

        # notify per SMS
        for phone in self.phones:
            try:
                self.notify_email('%s@t-mobile-sms.de' % phone, subject, message)
            except:
                log.error(
                    'SMS notification failed: %s [%s]',
                    phone, subject, exc_info=True)

        # notify per email
        for email in self.emails:
            try:
                self.notify_email(email, subject, message)
            except:
                log.error(
                    'Email notification failed: %s [%s]',
                    email, subject, exc_info=True)

    def notify_cityruf(self, pager, message):
        data = {'action': 'SendMessage',
                'class': '4',
                'language': 'de',
                'number': pager,
                'message': message[:79],
                'service': '1'}
        r = urllib2.urlopen('http://inetgateway.emessage.de/cgi-bin/funkruf2.cgi',
                        urllib.urlencode(data))
        r = r.read()
        assert 'Ihr Funkruf wurde angenommen.' in r

    def notify_email(self, to, subject, message):
        message = email.mime.text.MIMEText(message.encode('utf-8'), 'plain', 'utf-8')
        message['Subject'] = subject
        message['To'] = to
        try:
            username = self.config.get('mail', 'username')
        except ConfigParser.NoOptionError:
            username = None
        try:
            password = self.config.get('mail', 'password')
        except ConfigParser.NoOptionError:
            password = None
        mailer = zope.sendmail.mailer.SMTPMailer(
            self.config.get('mail', 'server'),
            username=username,
            password=password,
            force_tls=self.config.getboolean('mail', 'tls'))
        mailer.send('support@gocept.com', [to], message.as_string())


def get_support_number():
    """Returns the phone numbers of the current supporter if we're currently
    within a support time.
    """
    notifications = Notifications.fromMain()
    for phone in notifications.phones:
        print phone


def notify_support_main():
    notifications, options, args = Notifications.fromMain()
    notifications.notify(args[0], sys.stdin.read())
